//ovladani zavor Viessmann 5700, elektromagneticky prestavnik spinany pres rele, zvuk pres seriovou linku

#include "DFRobotDFPlayerMini.h"
#include "SoftwareSerial.h"

// senzor              0   1  2  3
//                     A   B  C  D
byte Senzor_PIN[] = { 19, 18, 9, 8};
unsigned int Senzor_IN[] =        { 10, 10, 10, 10};              //spozdeni akce pri prvnim najeti na cidlo
unsigned int Senzor_OUT[] =       { 2000, 1000, 1000, 2000};     //spozdeni akce po poslednim najeti na cidlo
struct Senzor
{
  byte  vstupPin;                 //pin pripojeni cidla
  byte  jednou;                   //1 - pouze najeti na cidlo, 0 - najeti i opusteni cidla
  byte  minule;
  byte  stav;                     //0 - neaktivni, 1 - najeti na cidlo po spozdeniIN, 2 - vyjezd z cidla po spozdeniOUT, 3 - oznaceni stavu mezi 1 a 2 (nastavuje se pouze ve vyhodnoceni)
  unsigned int spozdeniIN;
  unsigned int spozdeniOUT;
  unsigned long casMillis;
};
Senzor cidlo[4];

const int Led_B = 10;       //Bila I
const int Led_C1 = 11;      //Cervena 1 J
const int Led_C2 = 12;      //Cervena 2 K
const int Zavory_N = 3;     //Zavory nahoru L
const int Zavory_D = 14;    //Zavory dolu M

unsigned int blikB = 665;   //interval bile 40/minuta
unsigned int blikC = 999;   //interval cervene 60/minuta
byte zvukCervene = 1;           //poradi zvuku na SD karte
byte zvukHlasitost = 18;        //hlasitost zvuku
unsigned long CasReset = 30000;    //po uplynuti dojde k resetu jako pred vjezdem
unsigned long ImpulsZavory = 100;  //delka impulsu pro ovladani zavor
unsigned long CasZmenyBlik = 0;
unsigned long CasZavora = 0;
unsigned long casMillisRes = 0;
int pocitadlo = 0;
int smer = 0;

boolean Sviti_B = true;     //blika bila, pri startu blika
boolean Sviti_C = false;    //blika cervena
boolean Svetla_B = false;
boolean Svetla_C = false;
boolean setDole = false;    //zavory dolu
boolean setNahore = true;   //zavory nahoru, pri startu zvednout
boolean zDole = false;
boolean zNahore = false;

                              //F  E
SoftwareSerial mySoftwareSerial(6, 7); // RX, TX
DFRobotDFPlayerMini myDFPlayer;

void setup() {
  //Serial.begin(115200);

  for (int i = 0; i < 4; i++) {
    pinMode(Senzor_PIN[i], INPUT_PULLUP);
    cidlo[i].vstupPin = Senzor_PIN[i];
    cidlo[i].jednou = 0;
    cidlo[i].minule = 0;
    cidlo[i].stav = 0;
    cidlo[i].spozdeniIN = Senzor_IN[i];
    cidlo[i].spozdeniOUT = Senzor_OUT[i];
    cidlo[i].casMillis = 0;
  }
  pinMode(Led_B, OUTPUT);
  digitalWrite(Led_B, LOW);
  pinMode(Led_C1, OUTPUT);
  digitalWrite(Led_C1, LOW);
  pinMode(Led_C2, OUTPUT);
  digitalWrite(Led_C2, LOW);
  pinMode(Zavory_N, OUTPUT);
  digitalWrite(Zavory_N, LOW);
  pinMode(Zavory_D, OUTPUT);
  digitalWrite(Zavory_D, LOW);

  mySoftwareSerial.begin(9600);
  if (!myDFPlayer.begin(mySoftwareSerial)) {
    while (true);
  }
   
  myDFPlayer.setTimeOut(500); //Set serial communictaion time out 500ms
  myDFPlayer.volume(zvukHlasitost);  //Set volume value (0~30)
  myDFPlayer.EQ(DFPLAYER_EQ_NORMAL);
  myDFPlayer.outputDevice(DFPLAYER_DEVICE_SD);
  delay(1000);
  myDFPlayer.stop();

  CasZavora = millis();
}


void loop() {

  stavS(pocitadlo);
  vyhodnoceniS();
  Bila();
  Cervena();
  Dolu();
  Nahoru();

  pocitadlo++;
  if (pocitadlo >= 4) {
    pocitadlo = 0;
  }

}

void Bila () {
  if (Sviti_B) {
    digitalWrite(Led_C1, LOW);
    digitalWrite(Led_C2, LOW);
    digitalWrite(Led_B, Svetla_B);
    if ((millis() - CasZmenyBlik) > blikB)
    {
      CasZmenyBlik = millis();
      Svetla_B = !Svetla_B;
    }
  }
}

void Cervena () {
  if (Sviti_C) {
    digitalWrite(Led_B, LOW);
    digitalWrite(Led_C1, Svetla_C);
    digitalWrite(Led_C2, !Svetla_C);
    //Zvuk(!Svetla_C);
    if ((millis() - CasZmenyBlik) > blikC)
    {
      CasZmenyBlik = millis();
      Svetla_C = !Svetla_C;
    }
  }
}

void Zvuk (boolean hrat) {
  if (hrat) {
    myDFPlayer.loop(zvukCervene);
  } else {
    myDFPlayer.stop();
  }
}

void Tma () {
  digitalWrite(Led_C1, LOW);
  digitalWrite(Led_C2, LOW);
  digitalWrite(Led_B, LOW);
}

void Nahoru() {
  if (setNahore) {
    //Serial.print("nahoru "); Serial.println((millis() - CasZavora));
    if ((unsigned long) (millis() - CasZavora) < ImpulsZavory) {
      digitalWrite(Zavory_N, HIGH);
      zDole = false;
      //Serial.println(" zavory nahore high");
    } else {
      digitalWrite(Zavory_N, LOW);
      setNahore = false;
      zNahore = true;
      //zDole = false;
      //Serial.println(" zavory nahore low");
    }
  }
}

void Dolu() {
  if (setDole) {
    //Serial.print("dolu "); Serial.println((millis() - CasZavora));
    if ((unsigned long) (millis() - CasZavora) < ImpulsZavory) {
      digitalWrite(Zavory_D, HIGH);
      zNahore = false;
      //Serial.println(" zavory dole high");
    } else {
      digitalWrite(Zavory_D, LOW);
      setDole = false;
      zDole = true;
      //zNahore = false;
      Zvuk(false);      //vypnuti ceka na dohrani zvuku cca 500ms, proto je to az po uzavreni zavor
      //Serial.println(" zavory dole low");
    }
  }
}

void stavS (int index) {
  unsigned long currentMillis;
  currentMillis = millis();

  if (cidlo[index].stav == 0) {
    if ((cidlo[index].minule == 0) && (!digitalRead(cidlo[index].vstupPin))) {           //najeti na cidlo, plati pro vsechna cidla
      cidlo[index].minule = 1;                                                           //oznaceni prvniho najeti
      cidlo[index].casMillis = currentMillis;                                            //cas prvniho najeti na cidlo
    }
    if (cidlo[index].minule == 1) {
      if ((unsigned long) (currentMillis - cidlo[index].casMillis) < cidlo[index].spozdeniIN) {
        // do nothing
      } else {
        cidlo[index].stav = 1;                                                          //najeti na cidlo a uplynuti spozdeniIN
        cidlo[index].minule = 2;                                                        //oznaceni uplynuti spozdeniIN
        //cidlo[index].casMillis = currentMillis;                                       //cas stavu 1
        //Serial.print(index); Serial.println(" spusteni jednou");
      }
    }
  } else if ((cidlo[index].stav == 1) || (cidlo[index].stav == 3)) {                 //muze nastat az po najeti na cidlo a uplynuti spozdeniIN, pripadne po nastaveni na stav 3
    if (cidlo[index].jednou == 0) {                                                  //testovat i vyjezd z cidla
      if (!digitalRead(cidlo[index].vstupPin)) {                                     //pri kazdem najeti na cidlo se cas vynuluje, pokud se najeti vejde do intervalu spozdeniOUT
        cidlo[index].casMillis = currentMillis;
        //Serial.print(index); Serial.println(" vjezd vzdy");  Serial.println(cidlo[index].casMillis);
      } else {
        if ((unsigned long) (currentMillis - cidlo[index].casMillis) < cidlo[index].spozdeniOUT) {
          // do nothing
        } else {
          cidlo[index].stav = 2;                                                      //opusteni cidla a uplynuti spozdeniOUT
          //cidlo[index].minule = 0;
          //Serial.print(index); Serial.println(" spozdeniOUT"); Serial.println(currentMillis - cidlo[index].casMillis);
          cidlo[index].casMillis = currentMillis;                                     //cas stavu 2
        }
      }
    }
  }
}


void vyhodnoceniS () {
  if (smer == 0) {
    if ((cidlo[3].minule == 0) && (cidlo[0].minule >= 1)) {                                  //nastaveni pro obousmerny provoz, prvni dve cidla podle smeru budou reagovat jenom na najeti
      cidlo[0].jednou = 1;
      cidlo[1].jednou = 1;
      smer = 1;
      casMillisRes = millis();                                                              // nastaveni casu pro pripadne vraceni do vychoziho stavu
    }
    if ((cidlo[0].minule == 0) && (cidlo[3].minule >= 1)) {
      cidlo[3].jednou = 1;
      cidlo[2].jednou = 1;
      smer = 2;
      casMillisRes = millis();
    }
  }
  // Serial.print("smer "); Serial.println(smer);
  if (smer == 1) {
    if ((cidlo[1].stav == 1) && (zNahore)) {         //vjezd na prostredni cidlo, zavory nahore. spusteni zavor dolu a vypnuti zvuku, dal blikani cervene
      if ((setDole == false) && (zNahore)) {
        CasZavora = millis();
        cidlo[1].stav = 3;                          //oznaceni stavu mezi 1 a 2, pokud by se nedal, dochazelo by k opetovnemu spousteni podminky stav == 1
        //Serial.println("smer 1, vjezd cidlo 1 stav 1, zavory dolu");
      }
      setDole = true;
    }
    if ((cidlo[3].stav == 1) && (zDole)) {          //vjezd na krajni cidlo, zavory dole. zvednuti zavor nahoru a tma
      if ((setNahore == false) && (zDole)) {
        //Serial.println("smer 1, vjezd cidlo 3 stav 1, zavory nahoru");
        CasZavora = millis();
        cidlo[3].stav = 3;
        Sviti_C = false;
        Sviti_B = false;
        Tma();
      }
      setNahore = true;
    }
  } else if (smer == 2) {
    if ((cidlo[2].stav == 1) && (zNahore)) {         //vjezd na prostredni cidlo, zavory nahore. spusteni zavor dolu a vypnuti zvuku, dal blikani cervene
      if ((!setDole) && (zNahore)) {
        CasZavora = millis();
        cidlo[2].stav = 3;
        //Serial.println("smer 2, vjezd cidlo 2 stav 1, zavory dolu");
      }
      setDole = true;
    }  else if ((cidlo[0].stav == 1) && (zDole)) {    //vjezd na krajbnicidlo, zavory dole. zvednuti zavor nahoru a tma
      if ((!setNahore) && (zDole)) {
        CasZavora = millis();
        //Serial.println("smer 2, vjezd cidlo 0 stav 1, zavory nahoru");
        cidlo[0].stav = 3;
        Sviti_C = false;
        Sviti_B = false;
        Tma();
      }
      setNahore = true;
    }
  }
  if (((cidlo[3].stav == 2) || (cidlo[0].stav == 2)) || (((unsigned long) (millis() - casMillisRes) > CasReset) && (casMillisRes > 0))) {      //opusteni cidla, nesviti cervena a bila. zacne blikat bila, vse se vynuluje. Stejne tak po uplynuti doby pro celkovy reset od prvniho najeti na cidlo
    //Serial.println("smer 1, vyjezd cidlo 3 stav 2 - vynulovani");
    Sviti_C = false;
    Sviti_B = true;
    smer = 0;
    casMillisRes = 0;
    for (int i = 0; i < 4; i++) {
      cidlo[i].minule = 0;
      cidlo[i].stav = 0;
      cidlo[i].jednou = 0;
      cidlo[i].casMillis = 0;
    }
  }
  if (((cidlo[3].stav == 1) || (cidlo[0].stav == 1)) && (Sviti_B)) {                        //vjezd na prvni cidlo, spusti se zvuk a blika cervena
    //Serial.println("smer 2, vjezd cidlo 3 stav 1");
    Sviti_C = true;
    Sviti_B = false;
    Zvuk(true);
  }
}
